@@ -5,8 +5,8 @@
 
 
 #define RAV1E_MAJOR 0
-#define RAV1E_MINOR 3
-#define RAV1E_PATCH 5
+#define RAV1E_MINOR 5
+#define RAV1E_PATCH 1
 
 
 #include <stddef.h>
@@ -275,6 +275,37 @@ typedef enum {
     RA_PIXEL_RANGE_FULL,
 } RaPixelRange;
 
+/**
+ * Rate Control Data
+ */
+typedef enum {
+    /**
+     * A Rate Control Summary Packet
+     *
+     * It is emitted once, after the encoder is flushed.
+     *
+     * It contains a summary of the rate control information for the
+     * encoding process that just terminated.
+     */
+    RA_RC_DATA_KIND_SUMMARY,
+    /**
+     * A Rate Control Frame-specific Packet
+     *
+     * It is emitted every time a frame is processed.
+     *
+     * The information contained is required to encode its matching
+     * frame in a second pass encoding.
+     */
+    RA_RC_DATA_KIND_FRAME,
+    /**
+     * There is no pass data available for now
+     *
+     * This is emitted if rav1e_rc_receive_pass_data is called more
+     * often than it should.
+     */
+    RA_RC_DATA_KIND_EMPTY,
+} RaRcDataKind;
+
 /**
  * Supported Transfer Characteristics
  *
@@ -426,6 +457,8 @@ typedef struct {
     uint16_t y;
 } RaChromaticityPoint;
 
+typedef void (*RaFrameOpaqueCb)(void*);
+
 /**
  * Encoded Packet
  *
@@ -450,6 +483,20 @@ typedef struct {
      * Frame type
      */
     RaFrameType frame_type;
+    /**
+     * User provided opaque data
+     */
+    void *opaque;
+    /**
+     * The reconstruction of the shown frame.
+     * This is freed automatically by rav1e_packet_unref().
+     */
+    RaFrame *rec;
+    /**
+     * The Reference Frame
+     * This is freed automatically by rav1e_packet_unref().
+     */
+    RaFrame *source;
 } RaPacket;
 
 #ifdef __cplusplus
@@ -488,6 +535,36 @@ void rav1e_data_unref(RaData *data);
  */
 RaConfig *rav1e_config_default(void);
 
+/**
+ * Setup a second pass rate control using the provided summary
+ *
+ * Passing NULL data resets the rate control settings.
+ *
+ * If additional data is required, pointer and len stay unchanged, otherwise
+ * they are updated.
+ *
+ * Return:
+ * 0 on success
+ * > 0 if the buffer has to be larger
+ * < 0 on failure
+ */
+int rav1e_config_set_rc_summary(RaConfig *cfg, const uint8_t **data, size_t *len);
+
+/**
+ * Request to emit pass data
+ *
+ * Set emit to 0 to not emit pass data, non-zero to emit pass data.
+ *
+ */
+void rav1e_config_set_emit_data(RaConfig *cfg, int emit);
+
+/**
+ * Set the display aspect ratio of the stream
+ *
+ * Needed for anamorphic video.
+ */
+void rav1e_config_set_sample_aspect_ratio(RaConfig *cfg, RaRational sample_aspect_ratio);
+
 /**
  * Set the time base of the stream
  *
@@ -558,13 +635,31 @@ void rav1e_config_unref(RaConfig *cfg);
  * Set a configuration parameter using its key and value as string.
  *
  * Available keys and values
- * - "quantizer": 0-255, default 100
+ * - "width": width of the frame, default 640
+ * - "height": height of the frame, default 480
  * - "speed": 0-10, default 6
+ * - "threads": maximum number of threads to be used
  * - "tune": "psnr"-"psychovisual", default "psychovisual"
+ * - "quantizer": 0-255, default 100
+ * - "tiles": total number of tiles desired (0 denotes auto), default 0
+ * - "tile_rows": number of tiles horizontally (must be a power of two, overridden by tiles if present), default 0
+ * - "tile_cols": number of tiles vertically (must be a power of two, overridden by tiles if present), default 0
+ * - "min_quantizer": minimum allowed base quantizer to use in bitrate mode, default 0
+ * - "bitrate": target bitrate for the bitrate mode (required for two pass mode), default 0
+ * - "key_frame_interval": maximum interval between two keyframes, default 240
+ * - "min_key_frame_interval": minimum interval between two keyframes, default 12
+ * - "switch_frame_interval": interval between switch frames, default 0
+ * - "reservoir_frame_delay": number of temporal units over which to distribute the reservoir usage, default None
+ * - "rdo_lookahead_frames": number of frames to read ahead for the RDO lookahead computation, default 40
+ * - "low_latency": flag to enable low latency mode, default false
+ * - "enable_timing_info": flag to enable signaling timing info in the bitstream, default false
+ * - "still_picture": flag for still picture mode, default false
  *
  * Return a negative value on error or 0.
  */
-int rav1e_config_parse(RaConfig *cfg, const char *key, const char *value);
+int rav1e_config_parse(RaConfig *cfg,
+                       const char *key,
+                       const char *value);
 
 /**
  * Set a configuration parameter using its key and value as integer.
@@ -611,6 +706,17 @@ void rav1e_frame_unref(RaFrame *frame);
  */
 int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
 
+/**
+ * Register an opaque data and a destructor to the frame
+ *
+ * It takes the ownership of its memory:
+ * - it will relinquish the ownership to the context if
+ *   rav1e_send_frame is called.
+ * - it will call the destructor if rav1e_frame_unref is called
+ *   otherwise.
+ */
+void rav1e_frame_set_opaque(RaFrame *frame, void *opaque, RaFrameOpaqueCb cb);
+
 /**
  * Retrieve the first-pass data of a two-pass encode for the frame that was
  * just encoded. This should be called BEFORE every call to rav1e_receive_packet()
@@ -629,6 +735,66 @@ int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
  */
 RaData *rav1e_twopass_out(RaContext *ctx);
 
+/**
+ * Return the Rate Control Summary Packet size
+ *
+ * It is useful mainly to preserve space when saving
+ * both Rate Control Summary and Frame Packets in a single file
+ */
+size_t rav1e_rc_summary_size(const RaContext *ctx);
+
+/**
+ * Return the first pass data
+ *
+ * Call it after rav1e_receive_packet() returns a normal condition status:
+ * EncoderStatus::Encoded,
+ * EncoderStatus::Success,
+ * EncoderStatus::LimitReached.
+ *
+ * use rav1e_data_unref() to free the data.
+ *
+ * It will return a `RcDataKind::Summary` once the encoder is flushed.
+ */
+RaRcDataKind rav1e_rc_receive_pass_data(RaContext *ctx, RaData **data);
+
+/**
+ * Number of pass data packets required to progress the encoding process.
+ *
+ * At least that number of packets must be passed before the encoder can
+ * progress.
+ *
+ * Stop feeding-in pass data packets once the function returns 0.
+ *
+ * ``` c
+ * while (rav1e_rc_second_pass_data_required(ctx) > 0) {
+ *   int more = rav1e_rc_send_pass_data(ctx, &data, &len);
+ *   if (more > 0) {
+ *      refill(&data, &len);
+ *   } else if (more < 0) {
+ *     goto fail;
+ *   }
+ * }
+ * ```
+ *
+ */
+int32_t rav1e_rc_second_pass_data_required(const RaContext *ctx);
+
+/**
+ * Feed the first pass Rate Control data to the encoder,
+ * Frame-specific Packets only.
+ *
+ * Call it before receive_packet()
+ *
+ * If additional data is required, pointer and len stay unchanged, otherwise
+ * they are updated.
+ *
+ * Returns:
+ * - `0` on success,
+ * - `> 0` the amount of bytes needed
+ * - `< 0` on unrecoverable failure
+ */
+int rav1e_rc_send_pass_data(RaContext *ctx, const uint8_t **data, size_t *len);
+
 /**
  * Ask how many bytes of the stats file are needed before the next frame
  * of the second pass in a two-pass encode can be encoded. This is a lower
@@ -654,14 +820,19 @@ int rav1e_twopass_in(RaContext *ctx, uint8_t *buf, size_t buf_size);
  * Send the frame for encoding
  *
  * The function increases the frame internal reference count and it can be passed multiple
- * times to different rav1e_send_frame().
+ * times to different rav1e_send_frame() with a caveat:
+ *
+ * The opaque data, if present, will be moved from the Frame to the context
+ * and returned by rav1e_receive_packet in the Packet opaque field or
+ * the destructor will be called on rav1e_context_unref if the frame is
+ * still pending in the encoder.
  *
  * Returns:
  * - `0` on success,
  * - `> 0` if the input queue is full
  * - `< 0` on unrecoverable failure
  */
-RaEncoderStatus rav1e_send_frame(RaContext *ctx, const RaFrame *frame);
+RaEncoderStatus rav1e_send_frame(RaContext *ctx, RaFrame *frame);
 
 /**
  * Return the last encoder status
@@ -720,6 +891,31 @@ void rav1e_frame_fill_plane(RaFrame *frame,
                             ptrdiff_t stride,
                             int bytewidth);
 
+/**
+ * Extract a frame plane
+ *
+ * This is the reverse of rav1e_frame_fill_plane(), primarily used for
+ * extracting the source and reconstruction data from a RaPacket.
+ *
+ * Currently the frame contains 3 planes, the first is luminance followed by
+ * chrominance.
+ *
+ * The data is copied out of the frame for a single plane.
+ *
+ * frame: A frame provided inside a packet returned by rav1e_receive_packet()
+ * plane: The index of the plane starting from 0
+ * data: The destination for the data
+ * data_len: Length of the buffer
+ * stride: Plane line in bytes, including padding
+ * bytewidth: Number of bytes per component, either 1 or 2
+ */
+void rav1e_frame_extract_plane(const RaFrame *frame,
+                               int plane,
+                               uint8_t *data,
+                               size_t data_len,
+                               ptrdiff_t stride,
+                               int bytewidth);
+
 #ifdef __cplusplus
 } // extern "C"
 #endif // __cplusplus
